########################################################################
# Dialogs.tcl - Defines the various dialog-box procedures used by the
# 		rest of the program.
#
# Some of this code is based on code in the library accompanying
# the book Effective Tcl/Tk Programming, Mark Harrison and Michael
# McLennan, Addison Wesley, 1997.
########################################################################


option add *Confirm*mesg.wrapLength 4i widgetDefault


########################################################################
## USER CONFIGURATION: Uncomment one of the lines below to make the
## "help" button use a different browser, or change one of them to
## work with your setup.

set browserCommand "firefox file://$libDir/help/Introduction.htm"
#set browserCommand "gnome-terminal -e \"lynx $libDir/help/Introduction.htm\""
########################################################################


proc ShowAboutDialog { } {
	set aboutText ""
	append aboutText "f/Calc 1.00 (Tcl/Tk version)\n"
	append aboutText "\n"
	append aboutText "Copyright (c) 1999 by Warren Young\n"
	append aboutText "All rights reserved."
			
	set result [ Show2ButtonDialog $aboutText "About f/Calc" "OK" \
   			"License" ]
	if { $result == "2" } {
		ShowLicenseDialog
	}
}


proc ShowHelp { } {
	global browserCommand 
	if [ catch { open "|$browserCommand" r } error ] {
		ShowPromptDialog [ concat "Couldn't find the browser! " \
				"Please edit Dialogs.tcl to point f/Calc at " \
				"your browser." ] "Problem"
	}
}


proc ShowLicenseDialog { } {
    global confirmStatus

    set top [dialog_create "f/Calc License"]

    set info [dialog_info $top]
	scrollbar $info.sbar -command "$info.text yview"
	pack $info.sbar -side right -fill y
	text $info.text -wrap word -yscrollcommand "$info.sbar set"
	pack $info.text -side left -expand yes -fill both
	
    set cntls [dialog_controls $top]
    button $cntls.ok -text "OK" -command {set confirmStatus 1}
    pack $cntls.ok -side left -expand yes
    focus $cntls.ok

	$info.text configure -state disabled

	$info.text tag configure normal -spacing1 6p \
			-font -*-helvetica-medium-r-normal--*-120-*

	$info.text tag configure heading -spacing1 0.2i \
			-font -*-helvetica-bold-r-normal--*-120-*

	$info.text tag configure bold \
			-font -*-helvetica-bold-r-normal--*-120-*

	$info.text tag configure italic \
			-font -*-helvetica-medium-o-normal--*-120-*

	$info.text tag configure typewriter -wrap none \
			-font -*-courier-medium-r-normal--*-120-*
																	
	set fid [open "LICENSE" r]
	set contents [read $fid]
	close $fid
	$info.text configure -state normal
	$info.text delete 1.0 end
	$info.text insert end $contents "typewriter"
	$info.text configure -state disabled

    dialog_wait $top confirmStatus
    destroy $top
    return $confirmStatus
} 


proc ShowPromptDialog { mesg { title "Prompt" } { ok "OK" } } {
    global confirmStatus

    set top [dialog_create $title]

    set info [dialog_info $top]
    label $info.icon
    pack $info.icon -side left -padx 8 -pady 8
    label $info.mesg -text $mesg -justify left
    pack $info.mesg -side right -expand yes -fill both -padx 8 -pady 8

    set cntls [dialog_controls $top]
    button $cntls.ok -text $ok -command {set confirmStatus 1}
    pack $cntls.ok -side left -expand yes
    focus $cntls.ok

    dialog_wait $top confirmStatus
    destroy $top
    return $confirmStatus
}


proc Show2ButtonDialog { mesg { title "Prompt" } { button1 "OK" } \
		{ button2 "Cancel" } } {
    global confirmStatus

    set top [dialog_create $title]

    set info [dialog_info $top]
    label $info.icon
    pack $info.icon -side left -padx 8 -pady 8
    label $info.mesg -text $mesg -justify left
    pack $info.mesg -side right -expand yes -fill both -padx 8 -pady 8

    set cntls [dialog_controls $top]
    button $cntls.button1 -text $button1 -command {set confirmStatus 1}
    pack $cntls.button1 -side left -expand yes
    button $cntls.button2 -text $button2 -command {set confirmStatus 2}
    pack $cntls.button2 -side left -expand yes
    focus $cntls.button1

    dialog_wait $top confirmStatus
    destroy $top
    return $confirmStatus
}


proc dialog_create {class {win "auto"}} {
    if {$win == "auto"} {
        set count 0
        set win ".dialog[incr count]"
        while {[winfo exists $win]} {
            set win ".dialog[incr count]"
        }
    }
    toplevel $win -class $class

    frame $win.info
    pack $win.info -expand yes -fill both -padx 4 -pady 4

    frame $win.sep -height 2 -borderwidth 1 -relief sunken
    pack $win.sep -fill x -pady 4

    frame $win.controls
    pack $win.controls -fill x -padx 4 -pady 4

    wm title $win $class
    wm group $win .

    after idle [format {
        update idletasks
        wm minsize %s [winfo reqwidth %s] [winfo reqheight %s]
    } $win $win $win]

    return $win
}


proc dialog_info {win} {
    return "$win.info"
}


proc dialog_controls {win} {
    return "$win.controls"
}


proc dialog_wait {win varName} {
    dialog_safeguard $win

    set x [expr [winfo rootx .]+50]
    set y [expr [winfo rooty .]+50]
    wm geometry $win "+$x+$y"

    wm deiconify $win
    grab set $win

    vwait $varName

    grab release $win
    wm withdraw $win
}


bind modalDialog <ButtonPress> {
    wm deiconify %W
    raise %W
}


proc dialog_safeguard {win} {
    if {[lsearch [bindtags $win] modalDialog] < 0} {
        bindtags $win [linsert [bindtags $win] 0 modalDialog]
    }
}
