########################################################################
# DoF.tcl - Manages f/Calc's "Depth of Field" tab.
########################################################################


########################################################################
## Event Handlers
########################################################################

proc dof_UnitRadioClicked { } {
	global g_previousUnit
	global g_distanceUnit
	global g_subjectDistance

	if { $g_previousUnit != $g_distanceUnit } {
		set mm [ ToMillimeters $g_subjectDistance $g_previousUnit ]
		set g_subjectDistance \
				[ FromMillimeters $mm $g_distanceUnit ]
		set g_previousUnit $g_distanceUnit
		#RecalculateDoF
	}
}

proc dof_FocalLengthEntryChange { name index op } {
	RecalculateDoF
}

proc dof_SubjectDistanceEntryChange { name index op } {
	RecalculateDoF
}

proc dof_ApertureListChange { name1 name2 op } {
	RecalculateDoF
}

proc dof_CoCEntryChange { name index op } {
	RecalculateDoF
}


########################################################################
## Procedures
########################################################################

#### RecalculateDoF ####################################################
# Recalculates the depth-of-field

proc RecalculateDoF { } {
	global g_focalLength
	global g_subjectDistance
	global g_aperture
	global g_CoC
	
	global g_distanceUnit

	global dof_near
	global dof_far
	global dof_total

	set subjectDistance [ ToMillimeters $g_subjectDistance \
			$g_distanceUnit ] 

	set sane true
	if { $g_focalLength <= 0.0 } {
		set sane false
	} elseif { $subjectDistance < $g_focalLength } {
		set sane false
	} elseif { $g_aperture < 0.0 } {
		set sane false
	} elseif { $g_CoC < 0.0 } {
		set sane false
	}
	if { $sane == "false" } {
		set dof_near "n/a"
		set dof_far "n/a"
		set dof_total "n/a"
		return
	}

	# Do the calculations
	set dofRange [ DepthOfField $g_focalLength $g_aperture \
			$g_CoC $subjectDistance ]

	# Convert the calculated values to the user's unit of choice.
	set near [ FromMillimeters [ lindex $dofRange 0 ] \
			$g_distanceUnit ]
	set far [ FromMillimeters [ lindex $dofRange 1 ] \
			$g_distanceUnit ]

	# Build output strings
	set dof_near ""
	append dof_near "$near $g_distanceUnit"
	if { $far < 0.0 } {
		set dof_far "inifinity"
		set dof_total "infinite"
	} else {
		set dof_far ""
		append dof_far "$far $g_distanceUnit"
		set dof_total ""
		append dof_total [ expr $far - $near ] " $g_distanceUnit"
	}
}


########################################################################
## Tab setup
########################################################################

#### Add our tab to the top-level notebook
.noteBook add dof -label "Depth of Field" -underline 0
set dofTab [.noteBook subwidget dof]


#### Create the tab's controls
label $dofTab.promptLabel \
		-text [ concat "Calculate the depth of the field of acceptable " \
		"sharpness surrounding the focal plane:" ]
frame $dofTab.spacer1
tixLabelEntry $dofTab.focalLengthEntry -label "Focal length (mm): " \
		-options { \
			entry.width 10 
			label.width 22
			label.anchor e 
			entry.textVariable g_focalLength
		}
frame $dofTab.frame1
tixLabelEntry $dofTab.frame1.subjectDistanceEntry \
		-label "Subject distance: " -options { \
			entry.width 10 
			label.width 22
			label.anchor e 
			entry.textVariable g_subjectDistance
		}
radiobutton $dofTab.frame1.metersRadio -command dof_UnitRadioClicked \
		-text "m" -variable g_distanceUnit -value m 
radiobutton $dofTab.frame1.centimetersRadio -command dof_UnitRadioClicked \
		-text "cm" -variable g_distanceUnit -value cm
radiobutton $dofTab.frame1.feetRadio -command dof_UnitRadioClicked \
		-text "ft" -variable g_distanceUnit -value ft 
radiobutton $dofTab.frame1.inchesRadio -command dof_UnitRadioClicked \
		-text "in" -variable g_distanceUnit -value in
tixComboBox $dofTab.apertureList -label "Aperture: " -dropdown true \
		-editable true -variable g_aperture -options { \
			listbox.height 6
			label.width 22
			label.anchor e
		}
frame $dofTab.frame2
tixLabelEntry $dofTab.frame2.cocEntry -label "Circle of Confusion (mm): " \
		-options { \
			entry.width 10
			label.width 22
			label.anchor e
			entry.textVariable g_CoC
		}
tixComboBox $dofTab.frame2.suggestCoCList -label "Suggest CoC for: " \
		-dropdown true -editable false -variable g_filmFormat \
		-options { \
			listbox.height 6
			label.anchor w
		}
frame $dofTab.frame3
tixLabelEntry $dofTab.frame3.nearEntry -label "DoF near: " \
		-options { \
			entry.width 18
			label.anchor w
			entry.textVariable dof_near
		}
tixLabelEntry $dofTab.frame3.farEntry -label "DoF far: " \
		-options { \
			entry.width 18
			label.anchor w
			entry.textVariable dof_far
		}
tixLabelEntry $dofTab.frame3.totalEntry -label "Total: " \
		-options { \
			entry.width 18
			label.anchor w
			entry.textVariable dof_total
		}


#### Fill controls with defaults
$dofTab.frame1.metersRadio select
FillFList $dofTab.apertureList
tixSetSilent $dofTab.apertureList "1.8"
FillFormatList $dofTab.frame2.suggestCoCList
tixSetSilent $dofTab.frame2.suggestCoCList "35mm"
RecalculateDoF


#### Pack controls into tab
pack $dofTab.promptLabel -side top -anchor nw -pady 2
pack $dofTab.spacer1 -side top -anchor nw -pady 5
pack $dofTab.focalLengthEntry -side top -anchor nw -pady 2
pack $dofTab.frame1.subjectDistanceEntry -side left -anchor nw -pady 2
pack $dofTab.frame1.metersRadio -side left -anchor nw -pady 2
pack $dofTab.frame1.centimetersRadio -side left -anchor nw -pady 2
pack $dofTab.frame1.feetRadio -side left -anchor nw -pady 2
pack $dofTab.frame1.inchesRadio -side left -anchor nw -pady 2
pack $dofTab.frame1 -side top -anchor nw -pady 2
pack $dofTab.apertureList -side top -anchor nw -pady 2
pack $dofTab.frame2.cocEntry -side left -anchor nw -pady 2
pack $dofTab.frame2.suggestCoCList -side left -anchor nw -pady 2 -padx 8
pack $dofTab.frame2 -side top -anchor nw -pady 2
pack $dofTab.frame3.nearEntry -side left -anchor nw -pady 2
pack $dofTab.frame3.farEntry -side left -anchor nw -pady 2 -padx 8
pack $dofTab.frame3.totalEntry -side left -anchor nw -pady 2 -padx 8
pack $dofTab.frame3 -side top -anchor nw -pady 15


#### Disable output-only controls
set nearEntry [ $dofTab.frame3.nearEntry subwidget entry ]
set farEntry [ $dofTab.frame3.farEntry subwidget entry ]
set totalEntry [ $dofTab.frame3.totalEntry subwidget entry ]
$nearEntry configure -state disabled
$farEntry configure -state disabled
$totalEntry configure -state disabled


#### Set up "control changed" callbacks
trace variable g_focalLength w dof_FocalLengthEntryChange
trace variable g_subjectDistance w dof_SubjectDistanceEntryChange
trace variable g_aperture w dof_ApertureListChange
trace variable g_CoC w dof_CoCEntryChange
trace variable g_filmFormat w SuggestCoCListChange

