########################################################################
# Magnification.tcl - Manages f/Calc's "Magnification" tab.
########################################################################


########################################################################
## Globals
########################################################################

set magnification_previousUnit "m"


########################################################################
## Event Handlers
########################################################################

proc magnification_UnitRadioClicked { } {
	global magnification_previousUnit
	global g_distanceUnit
	global g_subjectDistance

	if { $magnification_previousUnit != \
			$g_distanceUnit } {
		set mm [ ToMillimeters $g_subjectDistance \
				$magnification_previousUnit ]
		set g_subjectDistance [ FromMillimeters $mm \
				$g_distanceUnit ]
		set magnification_previousUnit $g_distanceUnit
	}
}

proc magnification_SubjectDistanceEntryChange { name index op } {
	RecalculateMagnification
}

proc magnification_FocalLengthEntryChange { name index op } {
	RecalculateMagnification
}


########################################################################
## Procedures
########################################################################

#### RecalculateMagnification ##########################################
# Recalculates the magnification

proc RecalculateMagnification { } {
	global g_focalLength
	global g_subjectDistance
	
	global g_distanceUnit

	global magnification_result

	set subjectDistance [ ToMillimeters $g_subjectDistance \
			$g_distanceUnit ] 

	set sane true
	if { $g_focalLength <= 0.0 } {
		set sane false
	} elseif { $subjectDistance < 0.0 } {
		set sane false
	} elseif { $subjectDistance <= $g_focalLength } {
		set sane false
	}
	if { $sane == "false" } {
		set magnification_result "n/a"
		return
	}

	# Do the calculation and output the result
	set magnification_result [ Magnification $subjectDistance \
			$g_focalLength ]
}


########################################################################
## Tab setup
########################################################################

#### Add our tab to the top-level notebook
.noteBook add magnification -label "Magnification" -underline 2
set magnificationTab [.noteBook subwidget magnification]


#### Create the tab's controls
label $magnificationTab.promptLabel \
		-text [ concat "Calculate the subject magnification factor for " \
		"a lens of a given focal length, and the distance from\nthe" \
		"lens' optical center to the focal plane:" ] -justify left
frame $magnificationTab.spacer1
tixLabelEntry $magnificationTab.focalLengthEntry \
		-label "Focal length (mm): " -options { \
			entry.width 8 
			label.width 20
			label.anchor e 
			entry.textVariable g_focalLength
		}
frame $magnificationTab.frame1
tixLabelEntry $magnificationTab.frame1.subjectDistanceEntry \
		-label "Subject distance (So): " -options { \
			entry.width 8 
			label.width 20
			label.anchor e 
			entry.textVariable g_subjectDistance
		}
radiobutton $magnificationTab.frame1.metersRadio \
		-command magnification_UnitRadioClicked \
		-text "m" -variable g_distanceUnit -value m 
radiobutton $magnificationTab.frame1.centimetersRadio \
		-command magnification_UnitRadioClicked \
		-text "cm" -variable g_distanceUnit -value cm
radiobutton $magnificationTab.frame1.feetRadio \
		-command magnification_UnitRadioClicked \
		-text "ft" -variable g_distanceUnit -value ft 
radiobutton $magnificationTab.frame1.inchesRadio \
		-command magnification_UnitRadioClicked \
		-text "in" -variable g_distanceUnit -value in
tixLabelEntry $magnificationTab.magnificationEntry \
		-label "Magnification: " -options { \
			entry.width 18
			label.width 20
			label.anchor e
			entry.textVariable magnification_result
		}


#### Fill controls with defaults
$magnificationTab.frame1.metersRadio select
RecalculateMagnification


#### Pack controls into tab
pack $magnificationTab.promptLabel -side top -anchor nw -pady 2
pack $magnificationTab.spacer1 -side top -anchor nw -pady 5
pack $magnificationTab.focalLengthEntry -side top -anchor nw -pady 2
pack $magnificationTab.frame1.subjectDistanceEntry -side left -anchor nw
pack $magnificationTab.frame1.metersRadio -side left -anchor nw
pack $magnificationTab.frame1.centimetersRadio -side left -anchor nw
pack $magnificationTab.frame1.feetRadio -side left -anchor nw
pack $magnificationTab.frame1.inchesRadio -side left -anchor nw
pack $magnificationTab.frame1 -side top -anchor nw -pady 2
pack $magnificationTab.magnificationEntry -side top -anchor nw -pady 2


#### Disable output-only control
set magnificationEntry [ $magnificationTab.magnificationEntry \
		subwidget entry ]
$magnificationEntry configure -state disabled


#### Set up "control changed" callbacks
trace variable g_subjectDistance w \
		magnification_SubjectDistanceEntryChange
trace variable g_focalLength w magnification_FocalLengthEntryChange

