########################################################################
# SpotMeter.tcl - Manages f/Calc's "Spot Meter" tab.
########################################################################


########################################################################
## Event Handlers
########################################################################

proc spot_FilmFormatListChange { name1 name2 op } {
	RecalculateSpotMeter
}

proc spot_FocalLengthEntryChange { name index op } {
	RecalculateSpotMeter
}

proc spot_FramePercentEntryChange { name index op } {
	RecalculateSpotMeter
}


########################################################################
## Procedures
########################################################################

#### RecalculateSpotMeter ##############################################
# Recalculates the spot metering angle

proc RecalculateSpotMeter { } {
	global g_filmFormat
	global g_focalLength
	global spot_framePercent
	
	global spot_angle

	set sane true
	if { $g_focalLength <= 0.0 } {
		set sane false
	} elseif { $spot_framePercent < 0.0 } {
		set sane false
	}
	if { $sane == "false" } {
		set spot_angle "n/a"
		return
	}

	# Get the film size for this format
	set filmDimensions [ GetFilmDimensions $g_filmFormat ]
	set filmHeight [ lindex $filmDimensions 1 ]
	set filmWidth [ lindex $filmDimensions 0 ]
	set frameArea [ expr $filmHeight * $filmWidth ]

	# Do the calculation and output the result
	set spot_angle [ ToDegrees [ MeteringAngle $frameArea \
			$spot_framePercent $g_focalLength ] ]
	append spot_angle ""
}


########################################################################
## Tab setup
########################################################################

#### Add our tab to the top-level notebook
.noteBook add spot -label "Spot Meter" -underline 0
set spotTab [.noteBook subwidget spot]


#### Create the tab's controls
label $spotTab.promptLabel \
		-text [ concat "Convert a \"percent of frame\" measurement " \
		"of an in-camera spot meter to a \"degrees\"\nmeasurement," \
		"as projected through a given lens:" ] -justify left -justify left
frame $spotTab.spacer1
tixComboBox $spotTab.filmFormatList -label "Film format: " \
		-dropdown true -editable false -variable g_filmFormat \
		-options { \
			listbox.height 6
			label.anchor e
			label.width 18
			entry.width 10
		}
tixLabelEntry $spotTab.focalLengthEntry -label "Focal length (mm): " \
		-options { \
			entry.width 10 
			label.width 18
			label.anchor e 
			entry.textVariable g_focalLength
		}
tixLabelEntry $spotTab.framePercentEntry -label "Percent of frame: " \
		-options { \
			entry.width 10 
			label.width 18
			label.anchor e 
			entry.textVariable spot_framePercent
		}
frame $spotTab.spacer2
tixLabelEntry $spotTab.angleEntry \
		-label "Angle of View: " -options { \
			entry.width 16 
			label.width 18
			label.anchor e 
			entry.textVariable spot_angle
		}


#### Fill controls with defaults
FillFormatList $spotTab.filmFormatList
tixSetSilent $spotTab.filmFormatList "35mm"
set spot_framePercent "9.5"
RecalculateSpotMeter


#### Pack controls into tab
pack $spotTab.promptLabel -side top -anchor nw -pady 2
pack $spotTab.spacer1 -side top -anchor nw -pady 5
pack $spotTab.filmFormatList -side top -anchor nw -pady 2
pack $spotTab.focalLengthEntry -side top -anchor nw -pady 2
pack $spotTab.framePercentEntry -side top -anchor nw -pady 2
pack $spotTab.spacer2 -side top -anchor nw -pady 5
pack $spotTab.angleEntry -side left -anchor nw


#### Disable output-only control
set angleEntry [ $spotTab.angleEntry subwidget entry ]
$angleEntry configure -state disabled


#### Set up "control changed" callbacks
trace variable g_filmFormat w spot_FilmFormatListChange
trace variable g_focalLength w spot_FocalLengthEntryChange
trace variable spot_framePercent w spot_FramePercentEntryChange

