########################################################################
# fNumbers.tcl - Manages f/Calc's "f Numbers" tab.
########################################################################


########################################################################
## Event Handlers
########################################################################

proc fn_F2RadioClicked { } {
	global fNumbersTab

	set f1Entry [ $fNumbersTab.frame1.f1List subwidget entry ]
	set f1Arrow [ $fNumbersTab.frame1.f1List subwidget arrow ]
	set f2Entry [ $fNumbersTab.f2List subwidget entry ]
	set f2Arrow [ $fNumbersTab.f2List subwidget arrow ]
	set stopsEntry [ $fNumbersTab.frame1.stopsEntry subwidget entry ]
		
	$f1Entry configure -state normal
	$f1Arrow configure -state normal
	$f2Entry configure -state disabled
	$f2Arrow configure -state disabled
	$stopsEntry configure -state normal
	$fNumbersTab.roundOption configure -state disabled
}

proc fn_StopsRadioClicked { } {
	global fNumbersTab
	
	set f1Entry [ $fNumbersTab.frame1.f1List subwidget entry ]
	set f1Arrow [ $fNumbersTab.frame1.f1List subwidget arrow ]
	set f2Entry [ $fNumbersTab.f2List subwidget entry ]
	set f2Arrow [ $fNumbersTab.f2List subwidget arrow ]
	set stopsEntry [ $fNumbersTab.frame1.stopsEntry subwidget entry ]
		
	$f1Entry configure -state normal
	$f1Arrow configure -state normal
	$f2Entry configure -state normal
	$f2Arrow configure -state normal
	$stopsEntry configure -state disabled
	$fNumbersTab.roundOption configure -state normal
}

proc fn_F1ListChange { name index op } {
	global fn_calculateMode

	if { $fn_calculateMode == "stops" } {
		CalculateFStopDelta
	} else {
		CalculateNewFStop
	}
}

proc fn_F2ListChange { name index op } {
	global fn_calculateMode
	
	if { $fn_calculateMode == "stops" } {
		CalculateFStopDelta
	}
}

proc fn_StopsEntryChange { name1 name2 op } {
	global fn_calculateMode
	
	if { $fn_calculateMode == "f2" } {
		CalculateNewFStop
	}
}

proc fn_RoundOptionToggled { } {
	global fn_calculateMode

	if { $fn_calculateMode == "stops" } {
		CalculateFStopDelta
	} else {
		CalculateNewFStop
	}
}


########################################################################
## Procedures
########################################################################

#### CalculateFStopDelta ###############################################
# Calculates the number of stops between the two f numbers in the 
# "f lists".

proc CalculateFStopDelta { } {
	global fn_f1
	global fn_f2
	global fn_stops
	global fn_roundOption
	
	# Make sure f numbers are sane, and if so, do the calculation and
	# put the result in the "stops" field.
	if [ expr { $fn_f1 > 0.0 } && { $fn_f2 > 0.0 } ] {
		set tempStops [ ApertureDelta $fn_f1 $fn_f2 ]
		if { $fn_roundOption == "1" } {
			set tempStops [ RoundToFraction $tempStops 6 ]
		}
		set fn_stops $tempStops
	} else {
		set fn_stops "n/a"
	}
}


#### CalculateNewFStop #################################################
# Calculates an f stop from a base f stop plus a certain number of
# stops.  E.g., f/1.4 plus 2.5 stops is f/3.5.

proc CalculateNewFStop { } {
	global fn_f1
	global fn_f2
	global fn_stops
	global fn_roundOption
	
	# Calculate new f stop from old f stop and a delta
	set newAperture [ AdjustAperture $fn_f1 $fn_stops ]

	# Output the new f stop to the f2 field, if it's legal
	if [ expr $newAperture > 0.0 ] {
		set fn_f2 $newAperture
	} else {
		set fn_f2 "n/a"
	}
}


########################################################################
## Tab setup
########################################################################

#### Add our tab to the top-level notebook
.noteBook add fNumbers -label "f Numbers" -underline 2
set fNumbersTab [.noteBook subwidget fNumbers]


#### Create the tab's controls
label $fNumbersTab.promptLabel \
		-text "Supply two values to calculate the third:"
frame $fNumbersTab.spacer1
frame $fNumbersTab.frame1
tixComboBox $fNumbersTab.frame1.f1List -label "f1:" -dropdown true \
		-editable true -variable fn_f1 -options { \
			listbox.height 6
			label.width 4
			label.anchor e
			entry.width 14
		}
tixLabelEntry $fNumbersTab.frame1.stopsEntry -label "stops:" -options { \
			entry.width 8
			label.anchor w
			entry.textVariable fn_stops
		}
tixComboBox $fNumbersTab.f2List -label "f2:" -dropdown true \
		-editable true -variable fn_f2 -options { \
			listbox.height 6
			label.width 4
			label.anchor e
			entry.width 14
		}
label $fNumbersTab.calculateLabel -text "Calculate..."
frame $fNumbersTab.frame2
radiobutton $fNumbersTab.frame2.f2Radio -command fn_F2RadioClicked \
		-text "'f2' as 'f1' plus 'stops'" \
		-variable fn_calculateMode -value f2 
radiobutton $fNumbersTab.frame2.stopsRadio -command fn_StopsRadioClicked \
		-text "'stops' as 'f2' minus 'f1'" \
		-variable fn_calculateMode -value stops
frame $fNumbersTab.spacer2
checkbutton $fNumbersTab.roundOption -command fn_RoundOptionToggled \
		-text "Round calculated stops to nearest sixth of a stop" \
		-variable fn_roundOption


#### Set default control values
FillFList $fNumbersTab.frame1.f1List
tixSetSilent $fNumbersTab.frame1.f1List "1.8"
FillFList $fNumbersTab.f2List
tixSetSilent $fNumbersTab.f2List "1.8"
set fn_stops 0


#### Pack controls into tab
pack $fNumbersTab.promptLabel -side top -anchor nw -pady 2
pack $fNumbersTab.spacer1 -side top -anchor nw -pady 5
pack $fNumbersTab.frame1.f1List -side left -anchor nw -pady 2
pack $fNumbersTab.frame1.stopsEntry -side left -padx 6 -anchor nw -pady 2
pack $fNumbersTab.frame1 -side top -anchor nw -pady 2
pack $fNumbersTab.f2List -side top -anchor nw -pady 2 
pack $fNumbersTab.calculateLabel -side top -anchor nw -pady 2
pack $fNumbersTab.frame2.f2Radio -side top -anchor nw -pady 2
pack $fNumbersTab.frame2.stopsRadio -side top -anchor nw -pady 2
pack $fNumbersTab.frame2 -side top -anchor nw -padx 20 -pady 2
pack $fNumbersTab.spacer2 -side top -anchor nw -pady 5
pack $fNumbersTab.roundOption -side top -anchor nw -pady 2


#### Set up initial dialog state
$fNumbersTab.frame2.f2Radio select
fn_F2RadioClicked
$fNumbersTab.roundOption select


#### Set up "control changed" callbacks
trace variable fn_f1 w fn_F1ListChange
trace variable fn_f2 w fn_F2ListChange
trace variable fn_stops w fn_StopsEntryChange

